import pytest
import zengl


def test_invalid_image_read_multisample(ctx: zengl.Context):
    img = ctx.image((4, 4), 'rgba8unorm', samples=4)
    with pytest.raises(TypeError):
        img.read()


def test_invalid_image_read_cubemap(ctx: zengl.Context):
    img = ctx.image((4, 4), 'rgba8unorm', cubemap=True)
    with pytest.raises(TypeError):
        img.read()


def test_invalid_image_read_array(ctx: zengl.Context):
    img = ctx.image((4, 4), 'rgba8unorm', array=4)
    with pytest.raises(TypeError):
        img.read()



import pytest
import zengl


def test_invalid_buffer_parameters(ctx: zengl.Context):
    with pytest.raises(ValueError):
        ctx.buffer()

    with pytest.raises(TypeError):
        ctx.buffer('test')

    with pytest.raises(BufferError):
        ctx.buffer(memoryview(b'test')[::2])

    with pytest.raises(TypeError):
        ctx.buffer(100)

    with pytest.raises(ValueError):
        ctx.buffer(b'')

    with pytest.raises(ValueError):
        ctx.buffer(None)

    with pytest.raises(TypeError):
        ctx.buffer(size='999')

    with pytest.raises(ValueError):
        ctx.buffer(size=0)

    with pytest.raises(ValueError):
        ctx.buffer(size=-1)


def test_invalid_buffer_write_parameters(ctx: zengl.Context):
    buf = ctx.buffer(size=32)

    with pytest.raises(TypeError):
        buf.write('test')

    with pytest.raises(BufferError):
        buf.write(memoryview(b'test')[::2])

    with pytest.raises(TypeError):
        buf.write(10)

    with pytest.raises(TypeError):
        buf.write(None)

    with pytest.raises(ValueError):
        buf.write(b'123', 32 - 2)

    with pytest.raises(ValueError):
        buf.write(b'1', -1)

    with pytest.raises(ValueError):
        buf.write(b'1' * 33)

    with pytest.raises(TypeError):
        buf.write(b'1', offset=0.1)


def test_buffer_map_parameters(ctx: zengl.Context):
    buf = ctx.buffer(size=32)

    with pytest.raises(TypeError):
        buf.map(size=2.5)

    with pytest.raises(TypeError):
        buf.map(size='25')

    with pytest.raises(ValueError):
        buf.map(size=33)

    with pytest.raises(ValueError):
        buf.map(size=-1)

    with pytest.raises(ValueError):
        buf.map(size=0)

    with pytest.raises(TypeError):
        buf.map(size=1, offset=1.5)

    with pytest.raises(TypeError):
        buf.map(size=1, offset='15')

    with pytest.raises(ValueError):
        buf.map(offset=-1, size=1)

    with pytest.raises(ValueError):
        buf.map(offset=30, size=3)

    with pytest.raises(ValueError):
        buf.map(offset=64, size=1)

    buf.map(size=16, offset=8)
    with pytest.raises(RuntimeError):
        buf.map(size=16, offset=8)



import zengl

import utils

IMAGE_FORMATS = [
    'r8unorm', 'rg8unorm', 'rgba8unorm', 'r8snorm', 'rg8snorm', 'rgba8snorm', 'r8uint', 'rg8uint',
    'rgba8uint', 'r16uint', 'rg16uint', 'rgba16uint', 'r32uint', 'rg32uint', 'rgba32uint', 'r8sint', 'rg8sint',
    'rgba8sint', 'r16sint', 'rg16sint', 'rgba16sint', 'r32sint', 'rg32sint', 'rgba32sint', 'r16float', 'rg16float',
    'rgba16float', 'r32float', 'rg32float', 'rgba32float', 'rgba8unorm-srgb',
    'depth16unorm', 'depth24plus', 'depth24plus-stencil8', 'depth32float',
]


def test_texture_image_formats(ctx: zengl.Context):
    utils.clear_gl_error()
    for fmt in IMAGE_FORMATS:
        img = ctx.image((4, 4), fmt, texture=True)
        utils.assert_gl_error(fmt)
        ctx.release(img)
        utils.assert_gl_error()


def test_renderbuffer_image_formats(ctx: zengl.Context):
    utils.clear_gl_error()
    for fmt in IMAGE_FORMATS:
        img = ctx.image((4, 4), fmt, texture=False)
        utils.assert_gl_error(fmt)
        ctx.release(img)
        utils.assert_gl_error()
